package com.example.PhidgetResistanceInputExample;

import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.phidget22.*;

public class ResistanceInputExample extends Activity {

	ResistanceInput ch;
	SeekBar dataIntervalBar;

	Spinner rtdWireSetupSpinner;

	Toast errToast;

	int minDataInterval;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        //Hide device information and settings until one is attached
		LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
		settingsAndData.setVisibility(LinearLayout.GONE);

		//set data interval seek bar functionality
		dataIntervalBar = (SeekBar) findViewById(R.id.dataIntervalBar);
		dataIntervalBar.setOnSeekBarChangeListener(new dataIntervalChangeListener());

		//set rtdWireSetup spinner functionality
		rtdWireSetupSpinner = (Spinner) findViewById(R.id.rtdWireSetupSpinner);
		rtdWireSetupSpinner.setAdapter(new ArrayAdapter<RTDWireSetup>(this,
				android.R.layout.simple_spinner_item, RTDWireSetup.values()));
		rtdWireSetupSpinner.setOnItemSelectedListener(new rtdWireSetupChangeListener());

		LinearLayout rtdSection = (LinearLayout) findViewById(R.id.rtdWireSetupSection);
		rtdSection.setVisibility(LinearLayout.GONE);

        try
        {
        	ch = new ResistanceInput();

        	//Allow direct USB connection of Phidgets
			if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
                com.phidget22.usb.Manager.Initialize(this);

			//Enable server discovery to list remote Phidgets
			this.getSystemService(Context.NSD_SERVICE);
			Net.enableServerDiscovery(ServerType.DEVICE_REMOTE);

			ch.addAttachListener(new AttachListener() {
				public void onAttach(final AttachEvent attachEvent) {
				    AttachEventHandler handler = new AttachEventHandler(ch);
                    runOnUiThread(handler);
				}
			});

			ch.addDetachListener(new DetachListener() {
				public void onDetach(final DetachEvent detachEvent) {
                    DetachEventHandler handler = new DetachEventHandler(ch);
                    runOnUiThread(handler);

				}
			});

			ch.addErrorListener(new ErrorListener() {
				public void onError(final ErrorEvent errorEvent) {
					ErrorEventHandler handler = new ErrorEventHandler(ch, errorEvent);
					runOnUiThread(handler);

				}
			});

			ch.addResistanceChangeListener(new ResistanceInputResistanceChangeListener() {
				public void onResistanceChange(ResistanceInputResistanceChangeEvent resistanceChangeEvent) {
                    ResistanceInputResistanceChangeEventHandler handler = new ResistanceInputResistanceChangeEventHandler(ch, resistanceChangeEvent);
                    runOnUiThread(handler);
                }
			});

			ch.open();
        } catch (PhidgetException pe) {
	        pe.printStackTrace();
		}

    }

	private class dataIntervalChangeListener implements SeekBar.OnSeekBarChangeListener {
		@Override
    	public void onProgressChanged(SeekBar seekBar, int progress,
									  boolean fromUser) {
			if(fromUser) {
				try {
					TextView dataIntervalTxt = (TextView) findViewById(R.id.dataIntervalTxt);
					int dataInterval = progress + minDataInterval;
					dataIntervalTxt.setText(String.valueOf(dataInterval));
					ch.setDataInterval(dataInterval);
				} catch (PhidgetException e) {
					e.printStackTrace();
				}
			}
		}

		@Override
		public void onStartTrackingTouch(SeekBar seekBar) {}

		@Override
		public void onStopTrackingTouch(SeekBar seekBar) {}
	}

	private class rtdWireSetupChangeListener implements Spinner.OnItemSelectedListener {
		@Override
		public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id) {
			try {
				ch.setRTDWireSetup(RTDWireSetup.valueOf(parentView.getItemAtPosition(position).toString()));
			} catch (PhidgetException e) {
				e.printStackTrace();
			}
		}

		@Override
		public void onNothingSelected(AdapterView<?> parentView) {
			
		}

	}

	class AttachEventHandler implements Runnable {
    	Phidget ch;

		public AttachEventHandler(Phidget ch) {
			this.ch = ch;
		}

		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
			settingsAndData.setVisibility(LinearLayout.VISIBLE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);

			//Clear the error toast if a new ch is attached
			if (errToast != null) {
				errToast.cancel();
			}

			attachedTxt.setText("Attached");
			try {
				TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
				TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
				TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
				TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
				TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
				TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

				nameTxt.setText(ch.getDeviceName());
				serialTxt.setText(Integer.toString(ch.getDeviceSerialNumber()));
				versionTxt.setText(Integer.toString(ch.getDeviceVersion()));
				channelTxt.setText(Integer.toString(ch.getChannel()));
				hubPortTxt.setText(Integer.toString(ch.getHubPort()));
				labelTxt.setText(ch.getDeviceLabel());

				TextView dataIntervalTxt = (TextView) findViewById(R.id.dataIntervalTxt);
				dataIntervalTxt.setText(String.valueOf(((ResistanceInput)ch).getDataInterval()));

				minDataInterval = ((ResistanceInput)ch).getMinDataInterval();

				SeekBar dataIntervalBar = (SeekBar) findViewById(R.id.dataIntervalBar);
				dataIntervalBar.setProgress(((ResistanceInput)ch).getDataInterval() - minDataInterval);

				//Limit the maximum dataInterval on the SeekBar to 5000 so it remains usable
				if(((ResistanceInput)ch).getMaxDataInterval() >= 5000)
                    dataIntervalBar.setMax(5000 - minDataInterval);
				else
                    dataIntervalBar.setMax(((ResistanceInput)ch).getMaxDataInterval() - minDataInterval);

				if(ch.getDeviceID() == DeviceID.PN_TMP1200) {
					((LinearLayout)findViewById(R.id.rtdWireSetupSection)).setVisibility(LinearLayout.VISIBLE);

					//Set default selection to 4-wire
					rtdWireSetupSpinner.setSelection(2);
				}

			} catch (PhidgetException e) {
				e.printStackTrace();
			}

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }

    class DetachEventHandler implements Runnable {
    	Phidget ch;
    	
    	public DetachEventHandler(Phidget ch) {
    		this.ch = ch;
    	}
    	
		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
			settingsAndData.setVisibility(LinearLayout.GONE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);
			attachedTxt.setText("Detached");

			TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
			TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
			TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
			TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
			TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
			TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

			nameTxt.setText(R.string.unknown_val);
			serialTxt.setText(R.string.unknown_val);
			versionTxt.setText(R.string.unknown_val);
			channelTxt.setText(R.string.unknown_val);
			hubPortTxt.setText(R.string.unknown_val);
			labelTxt.setText(R.string.unknown_val);

			((LinearLayout)findViewById(R.id.rtdWireSetupSection)).setVisibility(LinearLayout.GONE);
			((TextView)findViewById(R.id.resistanceTxt)).setText("");

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }

    class ErrorEventHandler implements Runnable {
        Phidget ch;
        ErrorEvent errorEvent;

        public ErrorEventHandler(Phidget ch, ErrorEvent errorEvent) {
            this.ch = ch;
            this.errorEvent = errorEvent;
        }

        public void run() {
			 if (errToast == null)
				 errToast = Toast.makeText(getApplicationContext(), errorEvent.getDescription(), Toast.LENGTH_SHORT);

			 //replace the previous toast message if a new error occurs
			 errToast.setText(errorEvent.getDescription());
			 errToast.show();
        }
    }

	class ResistanceInputResistanceChangeEventHandler implements Runnable {
		Phidget ch;
		ResistanceInputResistanceChangeEvent resistanceChangeEvent;

		public ResistanceInputResistanceChangeEventHandler(Phidget ch, ResistanceInputResistanceChangeEvent resistanceChangeEvent) {
			this.ch = ch;
			this.resistanceChangeEvent = resistanceChangeEvent;
		}

		public void run() {
		    TextView resistanceTxt = (TextView)findViewById(R.id.resistanceTxt);
			resistanceTxt.setText(String.valueOf(resistanceChangeEvent.getResistance()));
		}
	}

    @Override
    protected void onDestroy() {
    	super.onDestroy();
    	try {
			ch.close();

		} catch (PhidgetException e) {
			e.printStackTrace();
		}

		//Disable USB connection to Phidgets
    	if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
            com.phidget22.usb.Manager.Uninitialize();
    }

}

